use strict;
use Bio::SeqIO;
use Bio::SeqFeature::Generic;
use DBI;
use POSIX;
use Getopt::Std;

################################################################################################
# SSRPoly Version 1.0
# SSRPoly_getSSRs.pl Version 1.0
# Copyright University of Queensland 2008
# SSRPoly was produced by Nikki Appleby, Jacqueline Batley and David Edwards
# For further information and contact details, please see README.txt
################################################################################################

my %args = ();
getopts("ho:a:fsrt",\%args) or &usage();

# h for help
&usage() if $args{h};

# o for organism
# will get parameters and any input files from the <organism> subdirectory
# assuming this script is in /scripts, the organism subdirectory will be at ../<organism>
if ($args{o}) {
    if (-e "../".$args{o}) {
	# that's fine
	&log("entered args: -o ".$args{o});
    } else {
	&usage();
    }
}

# analysisID if just a digit, optional clusterID if format is eg 1/1
my $ANAL_ID = 1;
my $CLUST_ID = 1;
if ($args{a}) {
    if ($args{a} =~ /(\d+)(\/\d+)?/) {
	$ANAL_ID = $1;
	my $clustID = $2;
	if (length($clustID) > 0) { $CLUST_ID = $clustID; }
    } else {
	&usage();
    }
}

# get parameters from parameter file
# can put anything in there in the form name=value and it will be picked up
# saves rework when things change
my %pars = {};
my $parameterFile = "../".$args{o}."/SSRPoly_params.txt";
&get_parameters();

#source, one of GenBank or GenPept or maybe Xref (where an xref link has been followed)
my $seqSource = 'GenBank';

# log files
my $runfile = makeFname($pars{"logDir"}, $pars{"runLogPrefix"}.get_time('fileFmt')."_SSRPoly_getSSRs.log", 1);
open(RUNFILE, ">", "$runfile") or die ("Cannot open file $runfile: $!");
my $errfile = makeFname($pars{"logDir"}, $pars{"errLogPrefix"}.get_time('fileFmt')."_SSRPoly_getSSRs.log", 1);
open(ERRFILE, ">", "$errfile") or die ("Cannot open file $errfile: $!");
&log("Log and error files successfully opened");
# log entered arguments
while (my ($parkey, $parval) = each %args) {
    &log("Entered argument $parkey = $parval");
}
# log parameters
while (my ($parkey, $parval) = each %pars) {
    # don't log anything that is a password
    if ($parkey !=~ /pwd/) {
	&log("Parameter $parkey = $parval");
    }
}

# Connect to the database.
my $dbh = DBI->connect("DBI:mysql:database=".$pars{"dbname"}.";".$pars{"dbhost"},
                         $pars{"dbusr"}, $pars{"dbpwd"},
                         {'RaiseError' => 1});
&log("database ".$pars{'dbname'}." open");
&prog("database ".$pars{'dbname'}." open");
my $fileToRead = "";
my $fullToRead = "";
my $fileOut = "";

my $currDate = get_time('date'); 
print $currDate,"\n";

my %seqs = ();
# preprepared sql statements
my $readSeqsql = "SELECT analysisID, clusterID, seqID, seqName, seq FROM t_sequence WHERE analysisID = $ANAL_ID AND clusterID = $CLUST_ID and length(seq) > 0";
my $readSeqsqlp = $dbh->prepare($readSeqsql);
my $writeSSRSeqsql = "INSERT INTO t_ssrSeq (analysisID, clusterID, seqID, seqSsrID, ssrtype, ssrStart, ssrEnd, ssrLen, score, ssrSeq, ssrCtgStart, motif, motifCount) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)";
my $writeSSRSeqsqlp = $dbh->prepare($writeSSRSeqsql);
my $seqContigsql = "SELECT q.seqID, q.contigID, q.consensusPosn, s.seq FROM t_contigSeq q INNER JOIN t_sequence s ON q.analysisID = s.analysisID AND q.clusterID = s.clusterID AND q.seqID = s.seqID WHERE q.analysisID = $ANAL_ID AND q.clusterID = $CLUST_ID";
my $seqContigsqlp = $dbh->prepare($seqContigsql);
my $ssrTypesql = "SELECT q.contigID, q.seqID, r.ssrCtgStart, r.ssrLen, r.motif, q.consensusPosn + q.qualClipStart AS qualStart, q.consensusPosn + q.qualClipEnd as qualEnd, r.motifCount, s.seq, q.consensusPosn FROM (t_sequence s INNER JOIN t_contigSeq q ON s.analysisID = q.analysisID AND s.clusterID = q.clusterID AND s.seqID = q.seqID) INNER JOIN t_ssrSeq r ON q.analysisID = r.analysisID AND q.clusterID = r.clusterID AND q.seqID = r.seqID WHERE s.analysisID = $ANAL_ID AND s.clusterID = $CLUST_ID ORDER BY contigID, ssrCtgStart";
my $ssrTypesqlp = $dbh->prepare($ssrTypesql);
my $ssrsql = "INSERT INTO t_ssr (analysisID, clusterID, contigID, ssrID, ssrStart, ssrEnd, ssrType, motif, motifCount, motifSeqCount, motifGroup, motifGroupName) VALUES (?,?,?,?,?,?,?,?,?,?,?,?)";
my $ssrsqlp = $dbh->prepare($ssrsql);
my $ssrSeqssrsql = "INSERT INTO t_ssrSeq_ssr (analysisID, clusterID, seqID, ssrID) VALUES (?,?,?,?)";
my $ssrSeqssrsqlp = $dbh->prepare($ssrSeqssrsql);


# clear table if -s option specified
if ($args{r}) {
    my $delSSRsqlp = $dbh->prepare("DELETE FROM t_ssrSeq WHERE analysisID = $ANAL_ID AND clusterID = $CLUST_ID");
    $delSSRsqlp->execute;
    $delSSRsqlp->finish;
    &log("All records deleted from t_ssrSeq, analysisID = $ANAL_ID, clusterID = $CLUST_ID");
    &prog("All records deleted from t_ssrSeq, analysisID = $ANAL_ID, clusterID = $CLUST_ID");
}
if ($args{t}) {
    my $delSSRGrpsqlp = $dbh->prepare("DELETE FROM t_ssr WHERE analysisID = $ANAL_ID AND clusterID = $CLUST_ID");
    $delSSRGrpsqlp->execute;
    $delSSRGrpsqlp->finish;
    my $delSSRGrpsqlp = $dbh->prepare("DELETE FROM t_ssrSeq_ssr WHERE analysisID = $ANAL_ID AND clusterID = $CLUST_ID");
    $delSSRGrpsqlp->execute;
    $delSSRGrpsqlp->finish;
    &log("All records deleted from t_ssr and t_ssrSeq_ssr, analysisID = $ANAL_ID, clusterID = $CLUST_ID");
    &prog("All records deleted from t_ssr and t_ssrSeq_ssr, analysisID = $ANAL_ID, clusterID = $CLUST_ID");
}
my $currSSRID = getMaxID("t_ssr", "ssrID")+1;

# GLOBAL variable for keeping ssr motifs and counts
my $motifGL = '';
my $motifCtGL = '';
my $motifGrpGL = '';
my $allComboNum = 0;
my %comboStr = ();

# temp store for ssrSeq - ssr data
# this so I can check that I don't put duplicates in
# which can happen in the following scenario
# acagacagacagGcagGcag	    2 ssrs following each other
# acagacagacagacagacag      next sequence just has one ssr that overlaps
# count whole thing as a single ssr
my %ssrSeqSsr = ();

# save all of the sequences and their start consensus positions within one ssr 
# used for assigning type
my %seqs = ();
my %seqStart = ();

#--------------------------------------------------------------------------------------------------------
# MAIN process 

my $tempFAFile = makeFname($pars{ssrDir}, $args{o}.".forSSR.fa", 0);
# only create fasta file if a -f flag entered
if ($args{f}) {
    &prog("Creating fasta file...");
    createFastaFileInput();
} else {
    &prog("Not creating fasta file (no -f option)");
}

# only run sputnik if a -s flag entered
my $resultFile = makeFname($pars{ssrDir}, $args{o}.".ssr.result", 0);
if ($args{s}) {
    &prog("Running sputnik...");
    # run sputnik with default parameters on that fasta file
    my $sputnikEXE = $pars{sputnikEXE};
    system("$sputnikEXE $tempFAFile > $resultFile");
} else {
    &prog("Skipped running sputnik (no -s option)");
}

# get a hash map of sequence contig IDs
&prog("Getting sequence-contig mapping...");
my %contigPosn = ();
getSeqContigHash();

# get a hash map of all 149 SSR types and make all combinations
&prog("Getting 149 SSR types and combinations...");
my %allComboGL = ();
my %groupName = ();
getSSRTypeHash();

# read in sputnik result line by line
my $ssrType = '';
if ($args{r}) {
    &prog("Reading sputnik result...");
    readSputnikResult();
} else {
    &prog("Skipping reading of sputnik result (no -r option)");
}

# read ssrs back in to assign polymorphic/monomorphic types
if ($args{t}) {
    &prog("Assigning type...");
    assignSSRType();
} else {
    &prog("Skipping assigning of type (no -t option)");
}

$readSeqsqlp->finish();
$writeSSRSeqsqlp->finish();
$seqContigsqlp->finish();
$ssrTypesqlp->finish();
$ssrsqlp->finish();
$ssrSeqssrsqlp->finish();

$dbh->disconnect();
&log("database closed");
&prog("database closed");
close (RUNFILE) or die( "\nCannot close file : $!");
close (ERRFILE) or die( "\nCannot close file : $!");

print "Finished.\n";

#--------------------------------------------------------------------------------------------------------

sub createFastaFileInput {

    my $readAnalID = 0;
    my $readClustID = 0;
    my $readSeqID = 0;
    my $readSeqName = '';
    my $readSeq = '';

    my $seq_fa = Bio::SeqIO->new(-format => 'fasta', -file => ">$tempFAFile"); 
    my $seqobj = undef;
    # extract sequences from the database
    $readSeqsqlp->execute() or &warn("read sequences sql: $readSeqsql");
    $readSeqsqlp->bind_columns( undef, \$readAnalID, \$readClustID, \$readSeqID, \$readSeqName, \$readSeq );
    while( $readSeqsqlp->fetch() ) {
	$readSeq =~ s/-/X/g;
        # print sequences to a fasta file with seqName_analysisID_clusterID_seqID as header
	$seqobj = Bio::PrimarySeq->new ( -seq => $readSeq, -id => "$readSeqName\_$readAnalID\_$readClustID\_$readSeqID");
        $seq_fa->write_seq($seqobj);
    }
}

#--------------------------------------------------------------------------------------------------------

sub getSeqContigHash {

    my $mapSeqID = 0;
    my $mapContigID = 0;
    my $mapCposn = 0;
    my $mapSeq = '';

    $seqContigsqlp->execute() or &warn("read sequences - contigs sql: $seqContigsql");
    $seqContigsqlp->bind_columns( \$mapSeqID, \$mapContigID, \$mapCposn, \$mapSeq );
    while( $seqContigsqlp->fetch() ) {
	$contigPosn{$mapSeqID} = $mapCposn;
    }
}

#--------------------------------------------------------------------------------------------------------

sub getSSRTypeHash {

    my @terms = ();
    my $t = 0;
    my $i = 0;
    my $j = 0;
    my $line = '';
    my $shiftStr = '';
    my $ssrTypFile = "SSRPoly_SSRs.txt";

    open(SSRFILE, "$ssrTypFile") or &die("Cannot open file $ssrTypFile: $!");
    while (defined($line = <SSRFILE>)) {
	chomp($line);
	if ($line) {
	    # line is eg 1,"AG","CT"
	    @terms = split(/,/,$line);
	    # store all shifts of $term[1] and $term[2] in %allComboGL
	    # ACG stores CGA, GAC, ACG against the number in $term[0]
	    for ($t = 1; $t <= 2; $t++) {
		for ($i = 0; $i <= length($terms[$t]); $i++) {
		    $shiftStr .= substr($terms[$t],$i,1);
		    $j = $i + 1;
		    while ($j != $i) {
			$shiftStr .= substr($terms[$t],$j,1);
			$j++;
			if ($j > length($terms[$t])) {
			    $j = 0;
			}
		    }
		    $allComboGL{$shiftStr} = $terms[0];
		    $shiftStr = '';
		}
	    }
	    # also save mapping between number and group name
	    $groupName{$terms[0]} = $terms[1];
	}
    }
    close(SSRFILE);


}

#--------------------------------------------------------------------------------------------------------

sub readSputnikResult {

    my $readLine = '';
    my $lineTypeCt = 0;
    my @headParts = ();
    my $ssrID = 1;
    my $ssrStart = 0;
    my $ssrEnd = 0;
    my $ssrLen = 0;
    my $ssrScore = 0;
    my $ssrSeq = '';
    my $prevLineType = 0;

    open(RESFILE, "$resultFile") or die ("Cannot open file $resultFile: $!");
    while ($readLine = <RESFILE>) {
	chomp($readLine);
	if ($readLine =~ /^>/) {
	    # starts > is a header line - reset variables
	    if ($prevLineType == 2) {
		writeSSRSeq($headParts[1], $headParts[2], $headParts[3], $ssrID, $ssrType, $ssrStart, $ssrEnd, $ssrLen, $ssrScore, $ssrSeq);
	    }
	    # extract analysisID, clusterID, seqID from header
	    @headParts = split(/_/, $readLine);
	    $ssrID = 0;
	    $ssrType = '';
	    $ssrStart = 0;
	    $ssrEnd = 0;
	    $ssrLen = 0;
	    $ssrScore = 0;
	    $ssrSeq = '';
	    $prevLineType = 0;
	} else {
	    if ($readLine =~ /(\w+)\s+(\d+)\s+:\s+(\d+)\s+\W+\s+length\s+(\d+)\s+score\s+(\d+)/) {
		if ($prevLineType == 2) {
		    writeSSRSeq($headParts[1], $headParts[2], $headParts[3], $ssrID, $ssrType, $ssrStart, $ssrEnd, $ssrLen, $ssrScore, $ssrSeq);
		}
		# one line after heading
		# eg. tetranucleotide 26 : 48 -- length 23 score 14
		$ssrType = $1;
		$ssrStart = $2;
		$ssrEnd = $3;
		$ssrLen = $4;
		$ssrScore = $5;
		$ssrSeq = '';
		$ssrID++;
		$prevLineType = 1;
	    } else {
		# two lines after heading - may be multiple lines
		# just sequence eg. AGGXAGGXAGGXAGGXAGGXA
		$ssrSeq .= $readLine;
		$prevLineType = 2;
	    }
	}
    }
}

#--------------------------------------------------------------------------------------------------------

sub assignSSRType {

    my $typContigID = undef;
    my $typSeqID = undef;
    my $typSsrID = undef;
    my $typStart = undef;
    my $typLen = undef;
    my $typMotif = undef;
    my $typQStart = undef;
    my $typQEnd = undef;
    my $typMotCount = undef;
    my $typSeq = undef;
    my $typCPosn = undef;
    my $typEnd = 0;
    my $ssrType = 'monomorphic';
    my $ssrStart = 0;
    my $ssrEnd = 0;
    my $ssrMot = '';
    my $ssrMotCt = 0;
    my %ssrMotifs = ();
    my $prevCtgID = 0;
    my $isFirst = 1;
    my $i = 0;

    $ssrTypesqlp->execute() or &warn("read ssrs sql: $ssrTypesql");
    $ssrTypesqlp->bind_columns( undef, \$typContigID, \$typSeqID, \$typStart, \$typLen, \$typMotif, \$typQStart, \$typQEnd, \$typMotCount, \$typSeq, \$typCPosn );
    while( $ssrTypesqlp->fetch() ) {
	if ($typContigID == 804) {
	    &prog("Stop here");
	}
	if ($typContigID != $prevCtgID) {
	    if ($isFirst == 1) {
		# first fetch, setup everything for next fetch
		$isFirst = 0;
		$ssrType = 'monomorphic';
		$ssrStart = $typStart;
		$ssrEnd = $typStart + $typLen - 1;
		$ssrMot = '';
		$ssrMotCt = $typMotCount;
		%ssrMotifs = ();
		$ssrMotifs{$typMotif} = 1;
		$typEnd = $typStart + $typLen - 1;
	    } else {
		# new contig - update previous ssr, then reset everything
		calcMotif(%ssrMotifs);
		$ssrType = calcType($ssrStart, $ssrEnd);
		writeSSR($prevCtgID, $ssrStart, $ssrEnd, $ssrType, $motifGL, $ssrMotCt, $motifCtGL, $motifGrpGL);
		$ssrType = 'monomorphic';
		$ssrStart = $typStart;
		$ssrEnd = $typStart + $typLen - 1;
		$ssrMot = '';
		$ssrMotCt = $typMotCount;
		%ssrMotifs = ();
		$ssrMotifs{$typMotif} = 1;
		$typEnd = $typStart + $typLen - 1;
	    }
	} else {
		$typEnd = $typStart + $typLen - 1;
		if ( ($typEnd > $ssrStart) && ($typStart < $ssrEnd) ) {
		    # ssrSeq is in the ssr, so add the ssrSeq details to the ssr
		    # if it is longer change ends and type is polymorphic
		    if ($typStart < $ssrStart) {
			$ssrStart = $typStart;
		    }
		    if ($typEnd > $ssrEnd) {
			$ssrEnd = $typEnd;
		    }
		    # motif
		    if (exists($ssrMotifs{$typMotif})) {
			$ssrMotifs{$typMotif}++;
		    } else {
			$ssrMotifs{$typMotif} = 1;
		    }
	    	} else {
		    # new ssr - update previous ssr, then move on to next one
		    calcMotif(%ssrMotifs);
		    $ssrType = calcType($ssrStart, $ssrEnd);
		    writeSSR($prevCtgID, $ssrStart, $ssrEnd, $ssrType, $motifGL, $ssrMotCt, $motifCtGL, $motifGrpGL);
		    $ssrType = 'monomorphic';
		    $ssrStart = $typStart;
		    $ssrEnd = $typStart + $typLen - 1;
		    $ssrMot = '';
		    $ssrMotCt = $typMotCount;
		    %ssrMotifs = ();
		    $ssrMotifs{$typMotif} = 1;
		    $typEnd = $typStart + $typLen - 1;
		}
	}
	    # ignore ssrSeqs that fall outside of the quality clipping
	    if ( ($typStart >= $typQStart) && ($typEnd <= $typQEnd) ) {
		$seqs{$typSeqID} = $typSeq;
		$seqStart{$typSeqID} = $typCPosn;
	    }
	$ssrSeqSsr{$typSeqID} = $currSSRID;
	$prevCtgID = $typContigID;
    }
    # clean up leftovers
    calcMotif(%ssrMotifs);
    $ssrType = calcType($ssrStart, $ssrEnd);
    writeSSR($prevCtgID, $ssrStart, $ssrEnd, $ssrType, $motifGL, $ssrMotCt, $motifCtGL, $motifGrpGL);
}

#--------------------------------------------------------------------------------------------------------

sub calcMotif {

    my (%motf) = @_;
    my %motf2 = %motf;
    my $singMotf1 = '';
    my $singMotf2 = '';
    my $firstLetter = '';
    my $midString = '';
    my $lastLetter = '';
    my %shiftMotf = ();
    my $motIdx = 0;
    my $maxVal = 0;
    my $maxMot = '';
    my $maxTot = 0;
    #my $retMotf = '';

    # set defaults in global variables
    $motifGL = '';
    $motifCtGL = '';
    $motifGrpGL = '';

    # worked example WE %motf contains { ACG => 1, CGA => 5, GAC => 2 }
    # WE isShifted looks up ACG, GAC, CGA in %allComboGL and gets a value of 9 for all of them
    # WE compares $singMotf1 = ACG and $singMotf2 = ACG, isShifted = true, $shiftMotf{9}{ACG} = 1
    # WE compares $singMotf1 = ACG and $singMotf2 = CGA, isShifted = true, $shiftMotf{9}{CGA} = 5
    # WE compares $singMotf1 = ACG and $singMotf2 = GAC, isShifted = true, $shiftMotf{9}{GAC} = 2
    for $singMotf1 ( keys(%motf) ) {
	for $singMotf2 ( keys(%motf2) ) {
	    if ( isShifted($singMotf1, $singMotf2) ) {
		unless (exists($shiftMotf{$allComboGL{$singMotf1}}{$singMotf2}) ) {
		    $shiftMotf{$allComboGL{$singMotf1}}{$singMotf1} = $motf{$singMotf1};
		}
	    }
	}
    }
    # WE - working through the loop -
    # WE $maxVal = 0, $maxMot = '', $maxTot = 0;
    # WE $motIdx = 9, $singMotf2 = ACG, $maxVal = 1, $maxMot = ACG, $maxTot = 1
    # WE $motIdx = 9, $singMotf2 = CGA, $maxVal = 5, $maxMot = CGA, $maxTot = 6
    # WE $motIdx = 9, $singMotf2 = GAC, $maxVal = 5, $maxMot = CGA, $maxTot = 8
    for $motIdx ( keys(%shiftMotf) ) {
	$motifGrpGL = $motIdx;
	$maxVal = 0;
	$maxMot = '';
	$maxTot = 0;
	for $singMotf2 ( keys(%{ $shiftMotf{$motIdx} }) ) {
	    if ($shiftMotf{$motIdx}{$singMotf2} > $maxVal) {
		$maxMot = $singMotf2;
		$maxVal = $shiftMotf{$motIdx}{$singMotf2};
	    }
	    $maxTot += $shiftMotf{$motIdx}{$singMotf2};
	}
	$motifGL .= ";$maxMot";
	$motifCtGL .= ";$maxTot";
    }
}

#--------------------------------------------------------------------------------------------------------

sub isShifted {

    my ($mot1, $mot2) = @_;
    my $retVal = 0; #false

    if ($allComboGL{$mot1} == $allComboGL{$mot2}) {
	$retVal = 1;
    }
    return $retVal;

}

#--------------------------------------------------------------------------------------------------------

sub calcType {

    my ($ssrSt, $ssrEn) = @_;
    my $sqKey = '';
    my $currSub = '';
    my $retType = 'monomorphic';

    for $sqKey (keys %seqs) {
	$currSub = substr($seqs{$sqKey}, $ssrSt - $seqStart{$sqKey}, $ssrEn - $ssrSt + 1);
	if ($currSub =~ /--/) {
	    $retType = 'POLYMORPHIC';
	}
    }
    %seqs = ();
    %seqStart = ();
    return $retType;

}

#--------------------------------------------------------------------------------------------------------

sub writeSSRSeq {
    # INSERT INTO t_ssrSeq (analysisID, clusterID, seqID, seqSsrID, ssrtype, ssrStart, ssrEnd, ssrLen, score, ssrSeq, ssrCtgStart, motif, motifCount) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?)

    my ($analID, $clustID, $seqID, $ssrID, $typ, $stt, $en, $len, $score, $seq) = @_;
    my $motif = '';
    my $motifCt = 0;
    my %convertLen = ( 'di' => 2, 'tri' => 3, 'tetra' => 4, 'penta' => 5 );

    $seq =~ s/X/-/g;
    # calculate the position of this SSR within the contig
    my $thisContigStart = $contigPosn{$seqID};
    my $thisSeqStart = $stt + $thisContigStart - 1;
    # calculate the motif and how many times it occurs
    $motif = $seq;
    $motifCt = 0;
    my $reportWord = 'di';
    if ($typ =~ /(\w+)nucleotide/) {
	$reportWord = $1;
    }
    my $reportLen = $convertLen{$reportWord};
    my $motifLen = 1000;
    while ($motifLen > $reportLen) {
	if ($motif =~ /([ACGTN-]+)\1/) {
	    $motif = $1;
	    $motifLen = length($motif);
	} else {
	    $motifLen = 0;
	}
    }
    while ($seq =~ /$motif/g) {
	$motifCt++;
    }
    $writeSSRSeqsqlp->execute($analID, $clustID, $seqID, $ssrID, $typ, $stt, $en, $len, $score, $seq, $thisSeqStart, $motif, $motifCt)
	or &warn("Error in writeSSRSeq: $analID, $clustID, $seqID, $ssrID, $typ, $stt, $en, $len, $score, $seq, $thisSeqStart, $motif, $motifCt: $!");
}

#----------------------------------------------------------------------------------------------------------

sub writeSSR {
    # INSERT INTO t_ssr (analysisID, clusterID, contigID, ssrID, ssrStart, ssrEnd, ssrType, motif, motifCount, motifSeqCount, motifGroup, motifGroupName) VALUES (?,?,?,?,?,?,?,?,?,?,?,?)

    my ($ctgID, $ssrSt, $ssrEn, $ssrTyp, $ssrMtf, $motCt, $motSqCt, $motGrp) = @_;

    $ssrMtf =~ s/^;//;
    $motSqCt =~ s/^;//;
    $ssrsqlp->execute($ANAL_ID, $CLUST_ID, $ctgID, $currSSRID, $ssrSt, $ssrEn, $ssrTyp, $ssrMtf, $motCt, $motSqCt, $motGrp, $groupName{$motGrp}) 
	    or &warn("write ssr sql: $ssrsql, params $ctgID, $currSSRID, $ssrSt, $ssrEn, $ssrTyp, $ssrMtf, $motCt, $motSqCt, $motGrp, ".$groupName{$motGrp});

    # write ssrSeq - ssr here
    my $seqi = 0;
    foreach $seqi (keys %ssrSeqSsr) {
	writeSSRSeqSSR($seqi);
    }
    %ssrSeqSsr = ();

    $currSSRID++;
}

#----------------------------------------------------------------------------------------------------------

sub writeSSRSeqSSR {
    # INSERT INTO t_ssrSeq_ssr (analysisID, clusterID, seqID, ssrID) VALUES (?,?,?,?)

    my ($seqID) = @_;
    $ssrSeqssrsqlp->execute($ANAL_ID, $CLUST_ID, $seqID, $currSSRID) or &warn("write ssrSeq-ssr sql: $ssrSeqssrsql, params ".$seqID.", $currSSRID");
}

#----------------------------------------------------------------------------------------------------------


sub getMaxID {

    my ( $tblname, $idfield ) = @_;
    my $maxId = 0;
    my $ssql = "SELECT MAX($idfield) FROM $tblname;";
    my $msth = $dbh->prepare($ssql);
    $msth->execute() or &warn("getMaxID sql: $ssql");
    my( $returnVal );
    $msth->bind_columns( \$returnVal );
    #print $msth->rows;
    while( $msth->fetch() ) {
	if ( $returnVal ne "" ) {
	    $maxId = $returnVal;
	}
    }
    $msth->finish();
    return $maxId;
}

#-----------------------------------------------------------------------------------------------------------

sub get_parameters () {
# read all parameters from the parameter file

    my $line = "";
    my @param = ();
    my $pos = -1;

    open(FIL_PAR, "$parameterFile") || die("Cannot open parameter file $parameterFile: $!");
    while (defined($line = <FIL_PAR>)) {
	chomp($line);
	if ($line) {
	    if ($line =~ /^#/) {
		# it's a comment line so do nothing
	    } else {
		@param = split(/=/, $line);
		@param[1] =~ s/"//g;
		$pars{@param[0]} = @param[1];
	    }
	}
    }
    close(FIL_PAR);

}

#--------------------------------------------------------------------------------------------------------------------------

sub makeDir {
# slash, directory, slash, sub directory, slash
# eg input C: and sub and you get C:\sub\ (windows slashes) 

    my ($dirUpper, $dirLower, $isRoot) = @_;
    my $dirResult = $dirUpper;
    if ($pars{"os"} eq "windows") {
	my $winSlash = "\\";
	$dirUpper =~ s/^\\//;
	$dirUpper =~ s/\\$//;
	$dirLower =~ s/^\\//;
	$dirLower =~ s/\\$//;
	if ($isRoot == 1) {
	    $dirResult = $dirUpper.$winSlash.$dirLower.$winSlash;
	} else {
	    $dirResult = $winSlash.$dirUpper.$winSlash.$dirLower.$winSlash;
	}
    }
    if ($pars{"os"} eq "linux") {
	$dirUpper =~ s/^\///;
	$dirUpper =~ s/\/$//;
	$dirLower =~ s/^\///;
	$dirLower =~ s/\/$//;
	$dirResult = "/".$dirUpper."/".$dirLower."/";
    }
    return $dirResult;

}

#-------------------------------------------------------------------------------------------------------------------------

sub makeFname {
# slash, directory, slash, filename, no slash at end
# eg input C: and myFile and you get C:\myFile (windows slashes)

    my ($dirUpper, $dirLower, $isRoot) = @_;
    my $dirResult = $dirUpper;
    if ($pars{"os"} eq "windows") {
	my $winSlash = "\\";
	$dirUpper =~ s/^\\//;
	$dirUpper =~ s/\\$//;
	$dirLower =~ s/^\\//;
	$dirLower =~ s/\\$//;
	if ($isRoot == 1) {
	    $dirResult = $dirUpper.$winSlash.$dirLower;
	} else {
	    $dirResult = $winSlash.$dirUpper.$winSlash.$dirLower;
	}
    }
    if ($pars{"os"} eq "linux") {
	$dirUpper =~ s/^\///;
	$dirUpper =~ s/\/$//;
	$dirLower =~ s/^\///;
	$dirLower =~ s/\/$//;
	$dirResult = "/".$dirUpper."/".$dirLower;
    }
    return $dirResult;

}

#------------------------------------------------------------------------------------------------------------------------

sub get_time {

    my ($retForm) = @_;
    my $retTime = "";

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year += 1900;
    $mon += 1;
    if ($mon <= 9) { $mon = "0$mon" };
    if ($mday <= 9) { $mday = "0$mday" };
    if ($hour <= 9) { $hour = "0$hour" };
    if ($min <= 9) { $min = "0$min" };
    if ($sec <= 9) { $sec = "0$sec" };
    
    if ($retForm eq 'time') {
	$retTime = "$hour:$min:$sec";
    } elsif ($retForm eq 'date') {
	$retTime = "$year-$mon-$mday";
    } elsif ($retForm eq 'fileFmt') {
	$retTime = "$year$mon$mday$hour$min$sec";
    } else {
	$retTime = "$year-$mon-$mday-$hour:$min:$sec";
    }
    return $retTime;

}

#------------------------------------------------------------------------------------------------------------------------

sub log {
    my $message = shift;
    print RUNFILE "LOG: ", get_time('time'), " $message\n";
}

sub debug {
    my $message = shift;
    print "$message\n";
}

sub prog {
    my $message = shift;
    print "$message\n";
}

sub die {
    my $message = shift;
    print "ERROR: ", get_time('time'), " $message\n";
}

sub warn {
    my $message = shift;
    print ERRFILE "WARNING: ", get_time('time'), " $message\n";
}

#------------------------------------------------------------------------------------------------------------------------

sub usage()
{
    print STDERR <<EOF;

    SSRPoly_getSSRs...

    Input: takes sequences from the t_sequence table from the database specified by organism.
    Prerequisite: The file SSRPoly_params.txt is in the species directory.
    Result: fills database, table t_ssr.

    usage: $0 -o <organism directory> [-h] [-c] -a [number or number/number] [-f] [-s]

    -h         	: this (help) message
    -o organism : organism directory
		  must be located ../here from this script
		  must contain SSRPoly_params.txt
    -a		: analysisID to find SSRs within, optionally analysisID/clusterID
		  default 1/1    
    -f		: create the temporary fasta file (it takes a long time so skip this flag
		  if it has been created previously)
    -s		: run the sputnik ssr finder (this also takes a long time)
    -r		: clear out the table t_ssrSeq and read in the sputnik result file
    -t		: assign ssr type as poly/monomorphic, clear out table t_ssr and 
		  t_ssrSeq_ssr and fill them again

EOF
    exit;
}

#------------------------------------------------------------------------------------------------------------------------
# END
#------------------------------------------------------------------------------------------------------------------------

