use strict;
use Bio::SeqIO;
use Bio::SeqFeature::Generic;
use DBI;
use Getopt::Std;

################################################################################################
# SSRPoly Version 1.0
# SSRPoly_parseAceToDb.pl Version 1.0
# Copyright University of Queensland 2008
# SSRPoly was produced by Nikki Appleby, Jacqueline Batley and David Edwards
# For further information and contact details, please see README.txt
################################################################################################

my %args = ();
getopts("hsr:o:",\%args) or &usage();

# h for help
&usage() if $args{h};

# o for organism
# will get parameters and any input files from the <organism> subdirectory
# assuming this scripts is in /scripts, the organism subdirectory will be at ../<organism>
if ($args{o}) {
    if (-e "../".$args{o}) {
	# that's fine
    } else {
	&usage();
    }
}

# get parameters from parameter file
# can put anything in there in the form name=value and it will be picked up
# saves rework when things change
my %pars = {};

# p for platform
my $parameterFile = "../".$args{o}."/SSRPoly_params.txt";
&get_parameters();

# s for singletons
my $CONTIG = 1;
my $fileExt = $pars{'contigFileExt'};
if ($args{s}) {
    $CONTIG = 0;
    $fileExt = $pars{'singletonFileExt'}
}

# r for restrict to a number
my $numToProcess = 1000000;
if ($args{r}) {
    $numToProcess = $args{r};
}

# files
my $fileToRead = "";
my $fileOut = "";
my $cap_infile = "";
my $cap_outfile = "";

# log files
my $runfile = makeFname($pars{"logDir"}, $pars{"runLogPrefix"}.get_time('fileFmt')."_SSRPoly_parseAceToDb.log", 1);
open(RUNFILE, ">", "$runfile") or die ("Cannot open file $runfile: $!");
my $errfile = makeFname($pars{"logDir"}, $pars{"errLogPrefix"}.get_time('fileFmt')."_SSRPoly_parseAceToDb.log", 1);
open(ERRFILE, ">", "$errfile") or die ("Cannot open file $errfile: $!");
&log("Log and error files successfully opened");
while (my ($parkey, $parval) = each %pars) {
    # don't log anything that is a password
    if ($parkey !=~ /pwd/) {
	&log("Parameter $parkey = $parval");
    }
}

# Connect to the database.
my $dbh = DBI->connect("DBI:mysql:database=".$pars{"dbname"}.";".$pars{"dbhost"},
                         $pars{"dbusr"}, $pars{"dbpwd"},
                         {'RaiseError' => 1});
&prog("database ".$pars{'dbname'}." open");

my $currDate = get_time('date');
my $tStart = get_time('time');
 
# Global ID variables
my $currAnalysisID = 0;
my $currClusterID = 0;
my $currSeqID = 0;
my $currContigID = 0;
getCurrentID();
# global variables to hold sequence alignments - global because used in many places
my $firstSeq = 0;
my $seqAnal = 0;
my $seqClust = 0;
my $seqCtg = 0;
my @seqAlign = ();
my @seqCount = ();
my @seqCtA = ();
my @seqCtC = ();
my @seqCtG = ();
my @seqCtT = ();
my @seqCtGap = ();
my @seqInform = ();
my $dirEntry = "";
my $progressReport = 20000;

my %contigRec = ();
# contigRec { name } =
#    clusterID
#    contigID
#    name
#    isContig
#    totSeqCount
#    totSnpCount
#    maxSeqCount
#    snpCount
#    indelCount
#    seqLength
#    consensusSeq

my %readRec = ();
#$readRec{ name } = {
#    clusterID	    => 0,
#    contigID	    => 0,
#    sequence	    => '',
#    complemented    => 'U',
#    consensusPosn   => 0,
#    clipStart	    => 0,
#    clipEnd	    => 0

# prepare queries once and use many times
my $writeSeqsqlp;
my $writeContigsqlp;
my $updateContigsqlp;
my $writeContigSeqsqlp;
prepareQueries();

# load all the files into memory
my @files = ();
opendir (FDIR, $pars{"readAceDir"}) or die "Failed to read from ".$pars{"readAceDir"}.": $!\n";
foreach $dirEntry (readdir(FDIR)) {
    if ($dirEntry =~ /$fileExt$/) {
	push (@files, $dirEntry);
    }
}
closedir (FDIR);

# main routine
my $maxLoop = 1;
foreach my $file (@files) {
    $cap_infile = makeFname($pars{"readAceDir"}, $file, 1);
    $cap_outfile = makeFname($pars{"logDir"}, $file.".txt", 1);
    if ($CONTIG) {
	writeAnalysis($file);
    	$maxLoop = capParse($maxLoop);
	writeAnalysisEnd();
    } else {
    	$maxLoop = singleParse($maxLoop);
    }
    if ($maxLoop > $numToProcess) {
	last;
    }
}
    
# Disconnect from the database.
terminatePreparedQueries();
$dbh->disconnect();
&log("database closed");
&prog("database closed");
close (RUNFILE) or die( "\nCannot close file : $!");
close (ERRFILE) or die( "\nCannot close file : $!");

print "\nFinished.";

#--------------------------------------------------------------------------------------------------------------------------

sub getCurrentID {

    my $retID = 1;

    my $ssql = "SELECT Max(analysisID) AS maxID FROM t_analysis;";
    my $sth = $dbh->prepare($ssql);
    $sth->execute() or &warn("ERROR getCurrentID from t_analysis error: $!");
    my ( $maxID ) = "";
    $sth->bind_columns( undef, \$maxID );
    while( $sth->fetch() ) {
	$retID = $maxID;
    }
    $sth->finish();
    $ssql = "SELECT Max(clusterID) AS maxID FROM t_analysis where analysisID = ?;";
    $sth = $dbh->prepare($ssql);
    $sth->execute($retID) or &warn("ERROR getting max clusterID from t_analysis error: $!");
    $maxID = "";
    $sth->bind_columns( undef, \$maxID );
    while( $sth->fetch() ) {
	$currClusterID = $maxID;
    }
    $sth->finish();
    # if it's a contig file then start a new analysis
    # if it's a file of singletons then associate them with the contigs that were just processed
    if ($CONTIG) {
	# if we are doing a new analysis of sequences and contigs then add one
	if ($args{a} ne 'only') {
	    $retID++;
	    $currClusterID++;
	} else {
	    # use the last analysisID when we want to replace data in the t_snp table
	    $ssql = "DELETE FROM t_snp where analysisID = ? AND clusterID = ?;";
	    $sth = $dbh->prepare($ssql);
	    $sth->execute($retID, $currClusterID) or &warn("ERROR clearing out t_snp: $!");
	    $sth->finish();
	    &log("All t_snp records for analysisID $retID clusterID $currClusterID DELETEd");
	}
    } else {
	$ssql = "SELECT Max(seqID) AS maxID FROM t_sequence where analysisID = ? and clusterID = ?;";
	$sth = $dbh->prepare($ssql);
	$sth->execute($retID, $currClusterID) or &warn("ERROR getting max seqID from t_sequence error: $!");
	$maxID = "";
	$sth->bind_columns( undef, \$maxID );
	while( $sth->fetch() ) {
	    $currSeqID = $maxID;
	}
	$sth->finish();
	$ssql = "SELECT Max(contigID) AS maxID FROM t_contig where analysisID = ? and clusterID = ?;";
	$sth = $dbh->prepare($ssql);
	$sth->execute($retID, $currClusterID) or &warn("ERROR getting max contigID from t_contig error: $!");
	$maxID = "";
	$sth->bind_columns( undef, \$maxID );
	while( $sth->fetch() ) {
	    $currContigID = $maxID;
	}
	$sth->finish();
    }
    $currAnalysisID = $retID;
    &log("t_analysis next analysis id $currAnalysisID cluster id $currClusterID");
#    return $retID;

}

#-------------------------------------------------------------------------------------------------------------------------

sub prepareQueries {

    my $writeSeqsql = "INSERT INTO t_sequence (analysisID, clusterID, seqID, seqName, seqDesc, seq) VALUES (?,?,?,?,?,?)";
    $writeSeqsqlp = $dbh->prepare($writeSeqsql);
    my $writeContigsql = "INSERT INTO t_contig (analysisID, clusterID, contigID, contigName, isContig, totalSeqCount, consensusLen, consensusSeq) VALUES (?,?,?,?,?,?,?,?)";
    $writeContigsqlp = $dbh->prepare($writeContigsql);
    my $writeContigSeqsql = "INSERT INTO t_contigSeq (analysisID, clusterID, seqID, contigID, consensusPosn, complement, qualClipStart, qualClipEnd)
		VALUES (?,?,?,?,?,?,?,?)";
    $writeContigSeqsqlp = $dbh->prepare($writeContigSeqsql);

}

#-------------------------------------------------------------------------------------------------------------------------

sub terminatePreparedQueries {

    $writeSeqsqlp->finish;
    $writeContigsqlp->finish;
    $writeContigSeqsqlp->finish;

}

#-------------------------------------------------------------------------------------------------------------------------

sub writeAnalysis {

    my ($fname) = @_;

    &log("$fname in progress...");
    if ($args{a} ne 'only') {
	my $ssql = "INSERT INTO t_analysis (analysisID, clusterID, analysisName, readDate, readStartTime, removeDate) VALUES (?,?,?,?,?,?)";
	my $sth = $dbh->prepare($ssql);
	$sth->execute($currAnalysisID, $currClusterID, $fname, $currDate, get_time('time'), $currDate) 
	    or &warn("ERROR t_analysis write error $currAnalysisID clusterID $currClusterID file name $fname ".$pars{"readAceFile"}.": $!");
	$sth->finish();
	&log("t_analysis rec saved: analysisID $currAnalysisID clusterID $currClusterID file name $fname ".$pars{"readAceFile"});
    }
    
}

#-------------------------------------------------------------------------------------------------------------------------

sub writeAnalysisEnd {

    if ($args{a} ne 'only') {
	my $ssql = "UPDATE t_analysis SET readEndTime=? WHERE analysisID=? AND clusterID=?";
	my $sth = $dbh->prepare($ssql);
	$sth->execute(get_time('time'), $currAnalysisID, $currClusterID) or &warn("t_analysis update error $currAnalysisID: $!");
	$sth->finish();
	&log("t_analysis rec updated: analysisID $currAnalysisID cluster $currClusterID");
    }
    
}

#-------------------------------------------------------------------------------------------------------------------------

sub writeSeq {
	# my $writeSeqsql = "INSERT INTO t_sequence (analysisID, clusterID, seqID, seqName, seqDesc, seq) VALUES (?,?,?,?,?,?)";

    my ($snm, $sd, $sq) = @_;
    my $ssql = "";

    #&log("writeSeq $snm, $sd");
    $currSeqID++;
    $writeSeqsqlp->execute($currAnalysisID, $currClusterID, $currSeqID, $snm, $sd, $sq) or &warn("ERROR t_sequence write error name $snm desc $sd: $!");
    #&log("t_sequence rec saved: analysisID $currAnalysisID seqID $currSeqID seqName $snm desc $sd");

}

#-------------------------------------------------------------------------------------------------------------------------

sub writeContig {
	# my $writeContigsql = "INSERT INTO t_contig (analysisID, clusterID, contigID, contigName, isContig, totalSeqCount, consensusLen, consensusSeq) 
	# VALUES (?,?,?,?,?,?,?,?)";

    my ($ctgnm, $sqtot, $conseq) = @_;
    my $ssql = "";
    
    # write a fresh contig record if you are analysing all sequences ($args{a} = 'all') or not looking for SNPs ($args{a} = 'no')
    # otherwise ($args{a} = 'only') just update the contig SNP info
    #&log("writeContig $currContigID $ctgnm");
    $writeContigsqlp->execute($currAnalysisID, $currClusterID, $currContigID, $ctgnm, $CONTIG, $sqtot, 
 	    $contigRec{seqLength}, $conseq) 
	or &warn("ERROR t_contig write error: analysisID $currAnalysisID clusterID $currClusterID contigID $currContigID seqID $currSeqID name $ctgnm total seqs $sqtot: $!");
    #&log("t_contig rec saved: analysisID $currAnalysisID clusterID $currClusterID contigID $currContigID name $ctgnm total seqs $sqtot");

}

#--------------------------------------------------------------------------------------------------------------------------------------------

sub writeContigSeq {
	# my $writeContigSeqsql = "INSERT INTO t_contigSeq (analysisID, clusterID, seqID, contigID, consensusPosn, complement, qualClipStart, qualClipEnd)
	# VALUES (?,?,?,?,?,?,?,?)";

    my ($singl, $cposn, $comp, $clpst, $clpen) = @_;
    my $ssql = "";

    #&log("writeContigSeq $cposn, $comp, $clpst, $clpen");
    $writeContigSeqsqlp->execute($currAnalysisID, $currClusterID, $currSeqID, $currContigID, $cposn, $comp, $clpst, $clpen) 
    or &warn("ERROR t_contigSeq write error: analysisID $currAnalysisID clusterID $currClusterID seqID $currSeqID contigID $currContigID consensusPosn $cposn complement $comp");
    #&log("t_contigSeq rec saved: analysisID $currAnalysisID clusterID $currClusterID seqID $currSeqID contigID $currContigID consensusPosn $cposn complement $comp");

}


#--------------------------------------------------------------------------------------------------------------------

sub capParse {
# Parse cap3 ace output in to the database

    my ( $procLoop ) = @_;
    my $cap_line = "";
    my @cap_array = ();
    my $sub_line = "";
    my @lineStart = ("AS", "CO", "BQ", "AF", "BS", "RD", "QA", "DS");
    my @arrLine = ();
    my $lineFirstWord = "";
    my $readName = '';

    &log("capParse");
    open (CAP_INFILE, "$cap_infile") || die("Cannot open cap_infile $cap_infile: $!");
    open(TXTFILE,">", $cap_outfile) or die ("cannot open file $cap_outfile: $!");

    my $i = 0;
    # if the first word matches the array above, matchIdx is the index value, otherwise it is -1
    my $matchIdx = -1;
    # booleans to indicate which section the current line is in - 1 while in a particular section, 0 while not
    my $inContig = 0;
    my $inRead = 0;
    my $inDesc = 0;
    my $afterDesc = 0;
    my $progressLinesRead = 0;
    my $progressLinesSaved = 0;
    my $progressOutput = 0;

    while (defined ($sub_line = <CAP_INFILE>)) {
	$progressLinesRead++;
	$progressOutput++;
	chomp($sub_line);
	@arrLine = split(/\s+/, $sub_line);
	$lineFirstWord = $arrLine[0];
	if ($lineFirstWord =~ /AS|CO|BQ|AF|BS|RD|QA|DS/) {
	    $progressLinesSaved++;
	    if ($inContig == 1) {
		$inContig = 0;
	    }
	    if ($inRead == 1) {
		$inRead = 0;
	    }
	    if ($inDesc == 1) {
		finaliseSeq($readName);
		$inDesc = 0;
		$afterDesc = 1;
	    }
	    if ($lineFirstWord =~ /^CO/) {
		if ($afterDesc == 1) {
		    finaliseContig();
		    $procLoop++;
		    $afterDesc = 0;
		}
		$currContigID++;
		$inContig = 1;
		$contigRec{name} = $arrLine[1];
		$contigRec{totSeqCount} = $arrLine[3];
		$contigRec{consensusSeq} = '';
		$contigRec{seqLength} = $arrLine[2];
		$contigRec{maxSeqCount} = 0;
		$contigRec{totSnpCount} = 0;
		$contigRec{snpCount} = 0;
		$contigRec{indelCount} = 0;
	    }
	    if ($lineFirstWord =~ /^AF/) {
		$readRec{$arrLine[1]}{complemented} = $arrLine[2];
		$readRec{$arrLine[1]}{consensusPosn} = $arrLine[3];
	    }
	    if ($lineFirstWord =~ /^RD/) {
		$inRead = 1;
		$readName = $arrLine[1];
		$readRec{$readName}{sequence} = '';
	    }
	    if ($lineFirstWord =~ /^QA/) {
		$readRec{$readName}{clipStart} = $arrLine[1];
		$readRec{$readName}{clipEnd} = $arrLine[2];
	    }
	    if ($lineFirstWord =~ /^DS/) {
		$inDesc = 1;
		$readRec{$readName}{descn} = substr($sub_line,3);
	    }
	} else {
	    if ($sub_line =~ /^[ACTGNactgn\*]/) {
		if ($inContig == 1) {
		    $contigRec{consensusSeq} .= $sub_line;
		    # remove non-sequence characters except * and -, then change * to -
		    $contigRec{consensusSeq} =~ s/[^A-Za-z\*-]//gi; 
		    $contigRec{consensusSeq} =~ s/\*/-/g;
		}
		if ($inRead == 1) {
		    $readRec{$readName}{sequence} .= $sub_line;
		    # remove non-sequence characters except * and -, then change * to -
		    $readRec{$readName}{sequence} =~ s/[^A-Za-z\*-]//gi; 
		    $readRec{$readName}{sequence} =~ s/\*/-/g;
		}
	    }
	}
	if ($procLoop >= $numToProcess) {
	    last;
	}
	if ($progressOutput == $progressReport) {
	    &prog(get_time()." At contigID $currContigID by number of lines [$progressLinesRead:$progressLinesSaved] [read:saved]");
	    &log("At contigID $currContigID by number of lines [$progressLinesRead:$progressLinesSaved] [read:saved]");
	    $progressOutput = 0;
	}
    }
    unless ($procLoop >= $numToProcess) {
	finaliseSeq($readName);
	finaliseContig();
    }
    close(TXTFILE);
    close(CAP_INFILE);
    return $procLoop;

}

#-------------------------------------------------------------------------------------------------------------------------

sub finaliseSeq {

    my ($readName) = @_;

    if ($args{a} ne 'only') {
	writeSeq($readName, $readRec{$readName}{descn}, $readRec{$readName}{sequence});
	writeContigSeq("N", $readRec{$readName}{consensusPosn}, $readRec{$readName}{complemented}, 
	    $readRec{$readName}{clipStart}, $readRec{$readName}{clipEnd});
    }
}

#-------------------------------------------------------------------------------------------------------------------------

sub finaliseContig {

    writeContig($contigRec{name}, $contigRec{totSeqCount}, $contigRec{consensusSeq});
    %contigRec = ();
    %readRec = ();
}

#------------------------------------------------------------------------------------------------------------------------

sub singleParse {

    my ( $procLoop ) = @_;
    my $sub_line = "";
    my @arrLine = ();
    my $seqStr = "";
    my $readName = "";
    my $descn = "";
    $contigRec{totSeqCount} = 1;
    my $progressLinesRead = 0;
    my $progressLinesSaved = 0;
    my $progressOutput = 0;

    &log("singleParse");
    open (SINGFILE, "$cap_infile") || die("Cannot open singleton file $cap_infile: $!");
    open(TXTFILE,">", $cap_outfile) or die ("cannot open singleton outfile $cap_outfile: $!");
    while (defined ($sub_line = <SINGFILE>)) {
	chomp($sub_line);
	if ($sub_line =~ /^>/) {
	    if ($seqStr ne "") {
		$currContigID++;
		$contigRec{seqLength} = length($seqStr);
		writeContig($readName, 1, $seqStr);
		writeSeq($readName, $descn, $seqStr);
		writeContigSeq("N", 1, "U", 1, length($seqStr));
		$procLoop++;
		$readName = "";
		$descn = "";
		$seqStr = "";
	    }
	    @arrLine = split(/\s+/, $sub_line);
	    $readName = $arrLine[0];
	    if ($sub_line =~ /$readName\s+(.*)/) {
		$descn = $1;
	    }
	    # if the version is tacked on the end of the accession number, remove it
	    if ($readName =~ /^>([^\.]*)(\.\d)?/) {
		$readName = $1;
	    }
	} else {
	   $seqStr .= $sub_line;
	}
	if ($procLoop >= $numToProcess) {
	    last;
	}
	if ($progressOutput == $progressReport) {
	    &prog(get_time()." At contigID $currContigID by number of lines [$progressLinesRead:$progressLinesSaved] [read:saved]");
	    &log("At contigID $currContigID by number of lines [$progressLinesRead:$progressLinesSaved] [read:saved]");
	    $progressOutput = 0;
	}
    }
    unless ($procLoop >= $numToProcess) {
	if ($seqStr ne "") {
	    $currContigID++;
	    writeContig($readName, 1, $seqStr);
	    writeSeq($readName, $descn, $seqStr);
	    writeContigSeq("N", 1, "U", 1, length($seqStr));
	    $procLoop++;
	}
    }
    close(TXTFILE);
    close(SINGFILE);
    return $procLoop;
}

#------------------------------------------------------------------------------------------------------------------------

sub get_parameters () {
# read all parameters from the parameter file

    my $line = "";
    my @param = ();
    my $pos = -1;

    open(FIL_PAR, "$parameterFile") || die("Cannot open parameter file $parameterFile: $!");
    while (defined($line = <FIL_PAR>)) {
	chomp($line);
	if ($line) {
	    if ($line =~ /^#/) {
		# it's a comment line so do nothing
	    } else {
		@param = split(/=/, $line);
		@param[1] =~ s/"//g;
		$pars{@param[0]} = @param[1];
	    }
	}
    }
    close(FIL_PAR);

}

#--------------------------------------------------------------------------------------------------------------------------

sub makeDir {
# slash, directory, slash, sub directory, slash
# eg input C: and sub and you get C:\sub\ (windows slashes) 

    my ($dirUpper, $dirLower, $isRoot) = @_;
    my $dirResult = $dirUpper;
    if ($pars{"os"} eq "windows") {
	my $winSlash = "\\";
	$dirUpper =~ s/^\\//;
	$dirUpper =~ s/\\$//;
	$dirLower =~ s/^\\//;
	$dirLower =~ s/\\$//;
	if ($isRoot == 1) {
	    $dirResult = $dirUpper.$winSlash.$dirLower.$winSlash;
	} else {
	    $dirResult = $winSlash.$dirUpper.$winSlash.$dirLower.$winSlash;
	}
    }
    if ($pars{"os"} eq "linux") {
	$dirUpper =~ s/^\///;
	$dirUpper =~ s/\/$//;
	$dirLower =~ s/^\///;
	$dirLower =~ s/\/$//;
	$dirResult = "/".$dirUpper."/".$dirLower."/";
    }
    return $dirResult;

}

#-------------------------------------------------------------------------------------------------------------------------

sub makeFname {
# slash, directory, slash, filename, no slash at end
# eg input C: and myFile and you get C:\myFile (windows slashes)

    my ($dirUpper, $dirLower, $isRoot) = @_;
    my $dirResult = $dirUpper;
    if ($pars{"os"} eq "windows") {
	my $winSlash = "\\";
	$dirUpper =~ s/^\\//;
	$dirUpper =~ s/\\$//;
	$dirLower =~ s/^\\//;
	$dirLower =~ s/\\$//;
	if ($isRoot == 1) {
	    $dirResult = $dirUpper.$winSlash.$dirLower;
	} else {
	    $dirResult = $winSlash.$dirUpper.$winSlash.$dirLower;
	}
    }
    if ($pars{"os"} eq "linux") {
	$dirUpper =~ s/^\///;
	$dirUpper =~ s/\/$//;
	$dirLower =~ s/^\///;
	$dirLower =~ s/\/$//;
	$dirResult = "/".$dirUpper."/".$dirLower;
    }
    return $dirResult;

}

#------------------------------------------------------------------------------------------------------------------------

sub get_time {

    my ($retForm) = @_;
    my $retTime = "";

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
    $year += 1900;
    $mon += 1;
    if ($mon <= 9) { $mon = "0$mon" };
    if ($mday <= 9) { $mday = "0$mday" };
    if ($hour <= 9) { $hour = "0$hour" };
    if ($min <= 9) { $min = "0$min" };
    if ($sec <= 9) { $sec = "0$sec" };
    
    if ($retForm eq 'time') {
	$retTime = "$hour:$min:$sec";
    } elsif ($retForm eq 'date') {
	$retTime = "$year-$mon-$mday";
    } elsif ($retForm eq 'fileFmt') {
	$retTime = "$year$mon$mday$hour$min$sec";
    } else {
	$retTime = "$year-$mon-$mday-$hour:$min:$sec";
    }
    return $retTime;

}

# ------------------------------------------------------------------------------

sub log {
    my $message = shift;
    print RUNFILE "LOG: ", get_time('time'), " $message\n";
}

sub debug {
    my $message = shift;
    print "$message\n";
}

sub prog {
    my $message = shift;
    print "$message\n";
}

sub die {
    my $message = shift;
    print "ERROR: ", get_time('time'), " $message\n";
}

sub warn {
    my $message = shift;
    print ERRFILE "WARNING: ", get_time('time'), " $message\n";
}

#------------------------------------------------------------------------------------------------------------------------

sub usage()
{
    print STDERR <<EOF;

    SSRPoly_parseAceToDb...

    usage: $0 -o <organism directory> [-h] [-p win/linux] [-s] [-r number of Contigs] [-a no] 

    -h         	: this (help) message
    -o organism : organism directory
		  must be located ../here from this script
		  must contain SSRPoly_params.txt
    -s          : if specified the files contain singletons, 
		  omit if the files contain contigs
    -r number   : restrict the run to just this number of contigs (mainly for 
		  testing), default parses the whole directory
    
EOF
    exit;
}

#------------------------------------------------------------------------------------------------------------------------
# That's All Folks!
#------------------------------------------------------------------------------------------------------------------------

